package cg;

/**
 * Simple or compound functions.
 */
public class Function {

	private static final int CONST		=    0;
	private static final int ID			=    1;
	
	private static final int SIN		= 1001;
	private static final int COS		= 1002;
	private static final int EXP		= 1003;
	private static final int LOG		= 1004;
	
	/**
	 * <code>+</code>
	 */
	public static final int PLUS		= 2001;
	
	/**
	 * <code>-</code>
	 */
	public static final int MINUS		= 2002;
	
	/**
	 * <code>*</code>
	 */
	public static final int MULT		= 2003;

	/**
	 * <code>/</code>
	 */
	public static final int DIV			= 2004;

	/**
	 * <code>^</code>
	 */
	public static final int POW			= 2005;

	/**
	 * Identity.
	 */
	public static final Function id 	= new Function(1, null);

	/**
	 * Sine.
	 */
	public static final Function sin 	= new Function(SIN, id);

	/**
	 * Cosine.
	 */
	public static final Function cos 	= new Function(COS, id);

	/**
	 * <code>e^x</code>
	 */
	public static final Function exp 	= new Function(EXP, id);

	/**
	 * Ln.
	 */
	public static final Function log 	= new Function(LOG, id);
	
	private int type;
	private double c;
	private Function f1;
	private Function f2;	
	
	private Function() {
		this(0.0);
	}
	
	/**
	 * Constant Function.
	 * @param c The constant.
	 */
	public Function(double c) {
		this.type = CONST;
		this.c = c;
	}
	
	private Function(int unaryType, Function f1) {
		this.type = unaryType;
		this.f1 = f1;		
	}
	
	/**
	 * Compound Function.
	 * @param binaryType Binary Operator.
	 * @param f1 First Operand.
	 * @param f2 Second Operand.
	 */
	public Function(int binaryType, Function f1, Function f2) {
		if (binaryType < 2001 || binaryType > 2005) {
			throw new RuntimeException("Not a binary type: " + binaryType);
		}
		this.type = binaryType;
		this.f1 = f1;
		this.f2 = f2;
	}
	
	/**
	 * Evaluates the function.
	 * @param t
	 * @return <code>f(t)</code>
	 */
	public double evaluate(double t) {
		switch (this.type) {
			case CONST:		return this.c;
			case ID:		return t;
			case SIN:		return Math.sin(this.f1.evaluate(t));
			case COS:		return Math.cos(this.f1.evaluate(t));
			case EXP:		return Math.exp(this.f1.evaluate(t));
			case LOG:		return Math.log(this.f1.evaluate(t));
			case PLUS:		return this.f1.evaluate(t) + this.f2.evaluate(t);
			case MINUS:		return this.f1.evaluate(t) - this.f2.evaluate(t);
			case MULT:		return this.f1.evaluate(t) * this.f2.evaluate(t);
			case DIV:		return this.f1.evaluate(t) / this.f2.evaluate(t);
			case POW:		return Math.pow(this.f1.evaluate(t), this.f2.evaluate(t));
			default: 		throw new RuntimeException();
		}
	}
}
